import { Callback, AsyncCallbackFn, CallbackFn, GraphMode, NodeFilterFn, NodeType, PartialConfig, PositionType, TraversalMode } from "./types";
import { WalkNode } from "./node";
declare class CallbacksBuilder<T extends CallbackFn, TUpper extends BaseWalkBuilder<T>> {
    private cbs;
    private source;
    private readonly callback;
    constructor(cbs: T[], source: TUpper);
    withExecutionOrder(order: number): this;
    withFilter(fn: NodeFilterFn): this;
    withFilters(...fn: NodeFilterFn[]): this;
    filteredByNodeTypes(...types: NodeType[]): this;
    filteredByKeys(...keys: string[]): this;
    filteredByPosition(position: PositionType): this;
    done(): TUpper;
}
declare abstract class BaseWalkBuilder<T extends CallbackFn> {
    protected _config: PartialConfig<T>;
    private globalFilters;
    resetConfig(): this;
    withConfig(config: PartialConfig<T>): this;
    withTraversalMode(traversalMode: TraversalMode): this;
    withGraphMode(graphMode: GraphMode): this;
    withRootObjectCallbacks(val: boolean): this;
    withRunningCallbacks(val: boolean): this;
    withConfiguredCallbacks(...callbacks: T[]): CallbacksBuilder<T, this>;
    withConfiguredCallback(callback: T): CallbacksBuilder<T, this>;
    withCallback(callback: Callback<T>): this;
    withGlobalFilter(fn: NodeFilterFn): this;
    withCallbacks(...callbacks: Callback<T>[]): this;
    getCurrentConfig(): PartialConfig<T>;
}
export declare class WalkBuilder extends BaseWalkBuilder<CallbackFn> {
    walk(target: any): void;
    walkStep(target: any): Generator<WalkNode>;
    withSimpleCallback(callback: CallbackFn): this;
    withSimpleCallbacks(...callbacks: CallbackFn[]): this;
}
export declare class AsyncWalkBuilder extends BaseWalkBuilder<AsyncCallbackFn> {
    walk(target: any): Promise<void>;
    walkStep(target: any): AsyncGenerator<WalkNode>;
    withParallelizeAsyncCallbacks(val: boolean): this;
    withSimpleCallback(callback: AsyncCallbackFn): this;
    withSimpleCallbacks(...callbacks: AsyncCallbackFn[]): this;
}
export {};
